"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.resolveBabelHelperImport = resolveBabelHelperImport;
exports.babelHelperResponseTransformer = babelHelperResponseTransformer;

var _fs = _interopRequireDefault(require("fs"));

var _util = require("util");

var _constants = require("../constants.js");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const readFile = (0, _util.promisify)(_fs.default.readFile);
const helperPrefix = '@babel/runtime';
const regeneratorPrefix = '@babel/runtime/regenerator';
const resolvedHelperPrefix = `${_constants.virtualFilePrefix}@babel/runtime`;
const resolvedRegeneratorPrefix = `${_constants.virtualFilePrefix}@babel/runtime/regenerator/index`;

async function resolveBabelHelperImport(importPath) {
  if (!importPath.startsWith(helperPrefix)) {
    return null;
  }

  if (importPath.startsWith(regeneratorPrefix)) {
    return `${importPath.replace(regeneratorPrefix, resolvedRegeneratorPrefix)}.js`;
  }

  return `${importPath.replace(helperPrefix, resolvedHelperPrefix)}.js`;
}

async function babelHelperResponseTransformer({
  url
}) {
  if (!url.startsWith(resolvedHelperPrefix)) {
    return null;
  }

  const resolvePath = url.replace(_constants.virtualFilePrefix, '');

  const resolvedPath = require.resolve(resolvePath);

  const body = await readFile(resolvedPath, 'utf-8');
  return {
    body,
    contentType: 'application/javascript',
    // cache for 1 week, since babel helpers can change
    cacheMaxAge: 604800
  };
}