"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.VirtualFileManager = exports.MAX_CACHE_AGE = void 0;
const MAX_CACHE_AGE = 31536000;
/**
 * @typedef {object} VirtualFile
 * @property {string} [body]
 * @property {string} [contentType]
 * @property {number} [cacheMaxAge]
 * @property {() => Promise<string>} [getBody]
 */

exports.MAX_CACHE_AGE = MAX_CACHE_AGE;

class VirtualFileManager {
  constructor() {
    /** @type {Map<string, VirtualFile>} */
    this.files = new Map();
  }
  /**
   * @param {string} key
   * @param {VirtualFile} file
   */


  addFile(key, file) {
    this.files.set(key, file);
  }

  async transformResponse({
    url
  }) {
    const file = this.files.get(url);

    if (!file) {
      return null;
    }

    const body = file.getBody ? await file.getBody : file.body;
    return { ...file,
      body
    };
  }

}

exports.VirtualFileManager = VirtualFileManager;